﻿using System;
using System.Linq;
using System.Net;
using System.Threading.Tasks;
using AsynchronousPrograming.POCOs;

namespace AsynchronousPrograming
{
    /// <summary>
    /// Vasim ukolem bude ziskat predpoved pocasi ze 2 ruznych API.
    /// K tomu navic zjistit pocet volnych pocitacu v knihovně na Filde (kdyby nahodou prselo).
    /// Pro usnadneni prace mate jiz predchystane ukazkove URL requesty na
    /// dilci sluzby. K dispozici mate take dalsi pomocne metody.
    /// 
    /// I.   Upravte metodu PerformRequest tak, aby pozadavek zasilala asynchrone
    /// 
    /// II.  Dokoncete metodu GetInfoAsync dle dokumentace
    /// 
    /// III. Implementujte metodu TestAsync, tato metoda vytvori 3 asynchroni 
    ///      requesty, ktere se hned po dokonceni asynchroni operace zpracuji
    ///      (pouzijte metodu ContinueWith(...), kterou zavolate na prislusnem 
    ///      Tasku) a vysledek se vypise na konzoli (viz predchystane metody v
    ///      regionu JsonProcessing)
    /// 
    /// </summary>
    class Program
    {
        #region weatherApiUrls
        /// <summary>
        /// Poskytovatel OpenWeather: max 100 pozadavku za minutu / url - v pripade prekroceni limitu vyuzijte OpenWeatherApi02
        /// </summary>
        private const string OpenWeatherApi01 = "http://api.openweathermap.org/data/2.5/weather?q=Brno&units=metric&APPID=d9fee349c9ee04bb29dea054bbf09555";
        private const string OpenWeatherApi02 = "http://api.openweathermap.org/data/2.5/weather?q=Brno&units=metric&APPID=6924b1d7b6bafd9214b4e6e8e5740969";

        /// <summary>
        /// Poskytovatel WeatherUnlocked: max 60 pozadavku za minutu / url - v pripade prekroceni limitu vyuzijte WeatherUnlockedApi02
        /// </summary>
        private const string WeatherUnlockedApi01 = "http://api.weatherunlocked.com/api/current/49.19,16.60?app_id=bcfb6773&app_key=222a3c7ed30850d0e86c708f01b6948a";
        private const string WeatherUnlockedApi02 = "http://api.weatherunlocked.com/api/current/49.19,16.60?app_id=3bfd31af&app_key=3f4a4eea942a3436a6f1b134e8244bff";

        /// <summary>
        /// Informace o ustredni knihovne FF
        /// Dokumentace API ve Swaggeru: https://ferda.ucn.muni.cz/occupationService/swagger/index.html
        /// </summary>
        private const string StudyroomOccupationApi = "https://ferda.ucn.muni.cz/occupationService/api/v2/localities/1301";
        #endregion

        static async Task Main(string[] args)
        {
            // TODO: Call test async...
            
        }    

        /// <summary>
        /// Shows current weather based on OpenWeather, WeatherUnlocked and FF free spaces
        /// </summary>
        /*public ... TestAsync()
        {
            TODO...

        }*/

        /// <summary>
        /// Gets weather info according to given parameter
        /// </summary>
        /// <param name="apiUrlString">Url string to make request to</param>
        /// <returns>Specific json with weather info</returns>
        private static async Task<string> GetInfoAsync(string apiUrlString)
        {
            var apiUrl = ParseUrl(apiUrlString);
            var apiName = apiUrlString.Split('.')[1];
            var weatherJson = string.Empty;

            // TODO...

            return weatherJson;
        }

        /// <summary>
        /// Performs request to API according to given apiUrl
        /// </summary>
        /// <param name="apiName">Short name of the api (used for logging)</param>
        /// <param name="apiUrl">Url to make request to</param>
        /// <returns>Specific json with weather info</returns>
        private static string PerformRequest(string apiName, Uri apiUrl)
        {
            Console.WriteLine($"Performed API call: {apiName}{Environment.NewLine}");
            using var handler = new HttpClientHandler { UseDefaultCredentials = true };
            using var httpClient = new HttpClient(handler);
            using var response = httpClient.GetAsync(apiUrl).Result;

            using var content = response.Content;
            var result = content.ReadAsStringAsync().Result;
            Console.WriteLine($"Received response from call: {apiName}{Environment.NewLine}");
            return result;
        }

        /// <summary>
        /// Parses given url into Uri
        /// </summary>
        /// <param name="apiUrlString">Url string to parse</param>
        /// <returns>Parsed url</returns>
        private static Uri ParseUrl(string apiUrlString)
        {
            var isUrlValid = Uri.TryCreate(apiUrlString, UriKind.Absolute, out var apiUrl)
                             && (apiUrl.Scheme == Uri.UriSchemeHttp || apiUrl.Scheme == Uri.UriSchemeHttps);
            if (!isUrlValid)
            {
                throw new UriFormatException("Given string is not valid url.");
            }
            return apiUrl ?? throw new UriFormatException("Given uri is null."); ;
        }

        #region JsonProcessing
        /// <summary>
        /// Prints weather report (from WeatherUnlockedApi) from given json
        /// </summary>
        /// <param name="weatherUnlockedJson">Json from WeatherUnlockedApi containing the weather report</param>
        private static void ProcessWeatherUnlockedJson(string weatherUnlockedJson)
        {
            var weatherUnlockedInfo =
                Newtonsoft.Json.JsonConvert.DeserializeObject<ProcessWeatherUnlockedResponse>(weatherUnlockedJson);
            Console.WriteLine(
                $"Brno forecast from WeatherUnlocked:{Environment.NewLine}{weatherUnlockedInfo?.wx_desc}{Environment.NewLine}temp: {weatherUnlockedInfo?.temp_c}°C{Environment.NewLine}humidity: {weatherUnlockedInfo?.humid_pct}%{Environment.NewLine}");
        }

        /// <summary>
        /// Prints weather report (from OpenWeatherApi) from given json
        /// </summary>
        /// <param name="openWeatherJson">Json from OpenWeatherApi containing the weather report</param>
        private static void ProcessOpenWeatherJson(string openWeatherJson)
        {
            var openWeatherInfo = Newtonsoft.Json.JsonConvert.DeserializeObject<ProcessOpenWeatherResponse2>(openWeatherJson);
            Console.WriteLine(
                $"Brno forecast from OpenWeather:{Environment.NewLine}{openWeatherInfo?.weather?.FirstOrDefault()?.description}{Environment.NewLine}temp: {openWeatherInfo?.main?.temp}°C{Environment.NewLine}humidity: {openWeatherInfo?.main?.humidity}%{Environment.NewLine}");
        }

        /// <summary>
        /// Prints avaliable computers (from OccupationServiceApi) from given json
        /// </summary>
        /// <param name="studyroomOccupationJson">Json from OccupationServiceApi containing the studyroom status</param>
        private static void ProcessOccupationJson(string studyroomOccupationJson)
        {
            var studyroomOccupationInfo = Newtonsoft.Json.JsonConvert.DeserializeObject<StudyroomOccupationPoco>(studyroomOccupationJson);
            Console.WriteLine(
                studyroomOccupationInfo?.status_code == 200
                    ? $"{studyroomOccupationInfo?.result:.name_en} has {studyroomOccupationInfo?.result?.available_computers} avaliable computers. {Environment.NewLine}"
                    : "<No data available>");
        }
        #endregion
    }
}
